#!/usr/bin/env python3
from subprocess import Popen, PIPE
import urllib.request
import socket
import sys
import re

template = """[Interface]
# This is the IP address of the server, it needs to be in the 10.0.0.0/8 network
Address = 10.0.0.1/8
# The UDP port used for Wireguard
ListenPort = 51820
# The server private key from above
PrivateKey = {}

# substitute eth0 in the following lines to match the Internet-facing interface
PostUp = iptables -A FORWARD -i %i -j ACCEPT; iptables -A FORWARD -o %i -j ACCEPT; iptables -t nat -A POSTROUTING -o eth0 -j MASQUERADE
PostDown = iptables -D FORWARD -i %i -j ACCEPT; iptables -D FORWARD -o %i -j ACCEPT; iptables -t nat -D POSTROUTING -o eth0 -j MASQUERADE

"""

peer_template = """[Peer]
# This is the public key for TH-100 with MAC {}
PublicKey = {}
# The IP of each TH-100 is "10.x.y.z" where x.y.z are the last 3 octets from the MAC address. The net mask should be always /32
AllowedIPs = {}

"""

th100ini_template = """[Wireguard]
Enable=1
Endpoint={}
ServerKey={}
"""

def genkeys():
    process = Popen(["wg", "genkey"], stdout=PIPE)
    (privout, err) = process.communicate()
    exit_code = process.wait()
    if exit_code != 0:
        raise Exception("Error generating private key")
    privkey = privout.decode('ascii').strip()
    process = Popen(["wg", "pubkey"], stdin=PIPE, stdout=PIPE)
    (pubout, err) = process.communicate(privout)
    exit_code = process.wait()
    if exit_code != 0:
        raise Exception("Error generating public key")
    pubkey = pubout.decode('ascii').strip()
    return privkey, pubkey

def validate_endpoint(endpoint):
    ip, port = endpoint.split(':')
    socket.inet_aton(ip)
    pnum = int(port)
    if pnum<=0 or pnum>=65535:
        raise Exception("Incorrect port")

def fetch_pubkey(mac):
    with urllib.request.urlopen('http://home.xakcop.com:8080/pubkey/{}'.format(mac)) as response:
       return response.read().decode('ascii')

def generate_server_conf(privkey, macs):
    conf = template.format(privkey)
    mac_regx = re.compile(r'^[0-9A-F]{2}\:[0-9A-F]{2}\:[0-9A-F]{2}\:[0-9A-F]{2}\:[0-9A-F]{2}\:[0-9A-F]{2}$', re.IGNORECASE)
    for mac in macs:
        if not mac_regx.match(mac):
            print("Skipping incorrect MAC: {}".format(mac))
            continue
        octets = mac.split(':')
        ip = "10.{}.{}.{}/32".format(int(octets[3], 16), int(octets[4], 16), int(octets[5], 16))
        pubkey = fetch_pubkey(mac)
        peer = peer_template.format(mac, pubkey, ip)
        conf += peer
    with open("wg0.conf", "w") as f:
        f.write(conf)
    print("Server config saved to wg0.conf")

def generate_th100ini(pubkey, endpoint):
    conf = th100ini_template.format(endpoint, pubkey)
    with open("th100.ini", "w") as f:
        f.write(conf)
    print("TH-100 config saved to th100.ini")

if __name__ == '__main__':
    if len(sys.argv) < 3:
        print("Usage: {} <file-with-macs> <endpoint>".format(sys.argv[0]))
        sys.exit(1)
    macs_file = sys.argv[1]
    privkey, pubkey = genkeys()
    with open(sys.argv[1], "r") as f:
        macs = f.read().split()
    endpoint = sys.argv[2]
    try:
        validate_endpoint(endpoint)
    except:
        print("Incorrect endpoint, expected <ip:port>")
        sys.exit(1)
    generate_server_conf(privkey, macs)
    generate_th100ini(pubkey, endpoint)
